@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2013 Red Hat, Inc.
+ * Copyright (C) 2008-2014 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2014 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -124,6 +124,7 @@ enum {
     UPDATE_PROPERTY,
     ENGINE_CHANGED,
     REQUEST_ENGINE,
+    SET_CONTENT_TYPE,
     LAST_SIGNAL,
 };
 
@@ -566,6 +567,18 @@ bus_input_context_class_init (BusInputContextClass *class)
             1,
             G_TYPE_STRING);
 
+    context_signals[SET_CONTENT_TYPE] =
+        g_signal_new (I_("set-content-type"),
+            G_TYPE_FROM_CLASS (class),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            bus_marshal_VOID__UINT_UINT,
+            G_TYPE_NONE,
+            2,
+            G_TYPE_UINT,
+            G_TYPE_UINT);
+
     text_empty = ibus_text_new_from_string ("");
     g_object_ref_sink (text_empty);
     lookup_table_empty = ibus_lookup_table_new (9 /* page size */, 0, FALSE, FALSE);
@@ -697,14 +710,15 @@ bus_input_context_property_changed (BusInputContext *context,
     if (context->connection == NULL)
         return TRUE;
 
-    GVariantBuilder *builder = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);
-    g_variant_builder_add (builder, "{sv}", property_name, value);
+    GVariantBuilder builder;
+    g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+    g_variant_builder_add (&builder, "{sv}", property_name, value);
     return bus_input_context_send_signal (context,
                                           "org.freedesktop.DBus.Properties",
                                           "PropertiesChanged",
                                           g_variant_new ("(sa{sv}as)",
                                                          "org.freedesktop.IBus",
-                                                         builder,
+                                                         &builder,
                                                          NULL),
                                           error);
 }
@@ -1149,6 +1163,14 @@ bus_input_context_service_set_property (IBusService     *service,
                                                    purpose,
                                                    hints);
 
+            if (context->has_focus) {
+                g_signal_emit (context,
+                               context_signals[SET_CONTENT_TYPE],
+                               0,
+                               context->purpose,
+                               context->hints);
+            }
+
             error = NULL;
             retval = bus_input_context_property_changed (context,
                                                          "ContentType",
@@ -2459,3 +2481,15 @@ bus_input_context_get_client (BusInputContext *context)
     g_assert (BUS_IS_INPUT_CONTEXT (context));
     return context->client;
 }
+
+void
+bus_input_context_get_content_type (BusInputContext *context,
+                                    guint           *purpose,
+                                    guint           *hints)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+    g_return_if_fail (purpose != NULL && hints != NULL);
+
+    *purpose = context->purpose;
+    *hints = context->hints;
+}
